import React from 'react';
import type { WithStyleProps, ComponentStyle } from '@instructure/emotion';
import type { OtherHTMLAttributes, PropValidators } from '@instructure/shared-types';
type TruncateListOwnProps = {
    /**
     * List of items in the truncated list
     */
    children?: React.ReactNode;
    /**
     * Sets the number of navigation items that are visible.
     * If not set, the list is not truncated.
     */
    visibleItemsCount?: number;
    /**
     * When there are list items hidden, an optional element
     * (dropdown menu, link, etc.) can be provided to display them
     * (renders at the end of the list).
     */
    renderHiddenItemMenu?: (hiddenChildren: Exclude<React.ReactNode, boolean | null | undefined>[]) => React.ReactElement;
    /**
     * Called whenever the navigation items are updated or the size of
     * the navigation changes. Passes in the `visibleItemsCount` as
     * a parameter.
     */
    onUpdate?: (visibleItemsCount: {
        visibleItemsCount: number;
    }) => void;
    /**
     * The spacing between list items (in 'rem', 'em' or 'px')
     */
    itemSpacing?: string;
    /**
     * Fix width of the Menu trigger (in 'rem', 'em' or 'px')
     */
    fixMenuTriggerWidth?: string;
    /**
     * The rate (in ms) the component responds to container resizing or
     * an update to one of its child items
     */
    debounce?: number;
    /**
     * Provides a reference to the underlying ul element
     */
    elementRef?: (element: HTMLUListElement | null) => void;
};
type PropKeys = keyof TruncateListOwnProps;
type AllowedPropKeys = Readonly<Array<PropKeys>>;
type TruncateListProps = TruncateListOwnProps & WithStyleProps<null, TruncateListStyle> & OtherHTMLAttributes<TruncateListOwnProps>;
type TruncateListStyle = ComponentStyle<'truncateList' | 'listItem' | 'menuTrigger'>;
type TruncateListState = {
    isMeasuring: boolean;
    menuTriggerWidth?: number;
};
declare const propTypes: PropValidators<PropKeys>;
declare const allowedProps: AllowedPropKeys;
export type { TruncateListProps, TruncateListStyle, TruncateListState };
export { propTypes, allowedProps };
//# sourceMappingURL=props.d.ts.map